"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const listr_1 = __importDefault(require("listr"));
const inquirer_1 = require("inquirer");
const service_1 = require("../../utils/service");
const login_1 = require("../../utils/login");
const firebase_1 = __importDefault(require("../../utils/firebase"));
const events_1 = require("events");
const ipfsClient = require('ipfs-http-client');
class Service extends command_1.Command {
    constructor() {
        super(...arguments);
        this.ipfsClient = ipfsClient('ipfs.app.mesg.com', '5001', { protocol: 'http' });
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { args, flags } = this.parse(Service);
            const password = flags.password
                ? flags.password
                : (yield inquirer_1.prompt([{ name: 'password', type: 'password', message: 'Type the password of your account' }])).password;
            let definition;
            let service;
            let runner;
            const { user } = yield login_1.loginFromCredential(this.config.configDir, password);
            const tasks = new listr_1.default([
                {
                    title: 'Compiling service',
                    task: () => __awaiter(this, void 0, void 0, function* () {
                        definition = yield service_1.compile(args.PATH, this.ipfsClient);
                    })
                },
                {
                    title: 'Creating service',
                    task: (ctx, task) => __awaiter(this, void 0, void 0, function* () {
                        const logs = yield this.deploy('service', definition, user.uid);
                        logs.on('data', x => task.output = x.message);
                        service = yield this.toPromise(logs);
                    })
                }
            ]);
            yield tasks.run();
            this.log('');
            this.log(`Service deployed with the hash ${service.hash}`);
            this.log('');
            const { deploy } = yield inquirer_1.prompt({ type: 'confirm', name: 'deploy', message: 'Would you like to start this service?' });
            if (deploy) {
                const env = yield this.getEnv(definition.configuration.env || []);
                yield new listr_1.default([
                    {
                        title: 'Starting service',
                        task: (ctx, task) => __awaiter(this, void 0, void 0, function* () {
                            const logs = yield this.deploy('runner', { serviceHash: service.hash, env }, user.uid);
                            logs.on('data', x => task.output = x.message);
                            runner = yield this.toPromise(logs);
                        })
                    }
                ]).run();
                this.log('');
                this.log(`Service started with the hash ${runner.hash}`);
                this.log('');
            }
            return firebase_1.default.firestore().terminate();
        });
    }
    getEnv(envs) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!envs.length)
                return [];
            this.log('Enter the value of the env variables:');
            const res = yield inquirer_1.prompt(envs.map((x) => ({
                name: x.split('=')[0],
                default: x.split('=')[1],
                type: 'input'
            })));
            return Object.keys(res).map(x => `${x}=${res[x]}`);
        });
    }
    deploy(type, definition, uid) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield firebase_1.default.firestore().collection('deployments').add({ type, uid, definition });
            const eventEmitter = new events_1.EventEmitter();
            let unsubscribeLogs;
            let unsubscribeDeployment;
            const clearListeners = () => {
                if (unsubscribeDeployment)
                    unsubscribeDeployment();
                if (unsubscribeLogs)
                    unsubscribeLogs();
            };
            unsubscribeLogs = res.collection('logs').onSnapshot(snapshots => {
                for (const change of snapshots.docChanges()) {
                    if (change.type !== 'added')
                        continue;
                    const data = change.doc.data();
                    if (data.level === 'error')
                        return eventEmitter.emit('error', new Error(data.message));
                    eventEmitter.emit('data', data);
                }
            }, error => eventEmitter.emit('error', error));
            unsubscribeDeployment = res.onSnapshot((snapshot) => __awaiter(this, void 0, void 0, function* () {
                const data = snapshot.data();
                if (!data.resourceRef)
                    return;
                const resource = yield data.resourceRef.get();
                eventEmitter.emit('end', resource.data().definition);
            }), error => eventEmitter.emit('error', error));
            eventEmitter.on('error', clearListeners);
            eventEmitter.on('end', clearListeners);
            return eventEmitter;
        });
    }
    toPromise(eventEmitter) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                eventEmitter.on('end', resolve);
                eventEmitter.on('error', reject);
            });
        });
    }
}
exports.default = Service;
Service.description = 'Deploy a service';
Service.flags = {
    password: command_1.flags.string({ description: "Password of your account" })
};
Service.args = [{
        name: 'PATH',
        description: 'Path or url of a service',
        default: './'
    }];
