"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const vault_1 = __importDefault(require("@mesg/vault"));
const file_1 = __importDefault(require("@mesg/vault/lib/store/file"));
const firebase_1 = __importDefault(require("firebase"));
const path_1 = require("path");
const firebase_2 = __importDefault(require("./firebase"));
const vaultKey = firebase_2.default.name;
const getVault = (configDir) => new vault_1.default(new file_1.default(path_1.join(configDir, 'credentials.json')));
exports.loginFromCredential = (configDir, password) => __awaiter(void 0, void 0, void 0, function* () {
    if (!exports.loggedIn(configDir))
        throw new Error('no account found, please run `mesg-cli login`');
    const vault = getVault(configDir);
    const credential = vault.get(vaultKey, password);
    try {
        return firebase_2.default.auth().signInWithEmailAndPassword(credential.email, credential.password);
    }
    catch (e) {
        throw new Error(e.message);
    }
});
exports.loggedIn = (configDir) => {
    const vault = getVault(configDir);
    return vault.contains(vaultKey);
};
exports.login = (configDir, email, password) => __awaiter(void 0, void 0, void 0, function* () {
    if (exports.loggedIn(configDir))
        throw new Error('Already logged in');
    const res = yield firebase_2.default.auth().signInWithEmailAndPassword(email, password);
    const vault = getVault(configDir);
    vault.set(vaultKey, firebase_1.default.auth.EmailAuthProvider.credential(email, password), password);
    return res;
});
exports.logout = (configDir) => __awaiter(void 0, void 0, void 0, function* () {
    if (!exports.loggedIn(configDir))
        return;
    const vault = getVault(configDir);
    vault.remove(vaultKey);
});
